#include "annotation_colors.hpp"

using namespace std;

DefaultColorMap::DefaultColorMap()
  : defaultColor(Color(0.0, 0.0, 0.0)) // set default color to black
{
}

DefaultColorMap::DefaultColorMap(const Color &c) 
  : defaultColor(c)
{
}

DefaultColorMap::DefaultColorMap(const DefaultColorMap &dcm)
  : defaultColor(dcm.defaultColor),
    cm(dcm.cm)
{
}

AnnotationColors::AnnotationColors()
{
}

AnnotationColors::AnnotationColors(const AnnotationColors &ac)
  : root_map(ac.root_map)
{
}

void AnnotationColors::clear()
{
  root_map.cm.clear();
  root_map.defaultColor = Color(0.0, 0.0, 0.0);
}

void AnnotationColors::setColor(Color &c)
{
  root_map.defaultColor = c;
}

Color AnnotationColors::color() 
{
  return root_map.defaultColor;
}

void AnnotationColors::appendTypeColor(const string &type, const Color &c)
{
  root_map.cm[type].defaultColor = c;
}

Color AnnotationColors::typeColor(const string &type)
{
  return root_map.cm[type].defaultColor;
}

void AnnotationColors::erase(const string &type)
{
  root_map.cm.erase(type);
}

void AnnotationColors::appendInstanceColor(const string &type, 
                                           const string &instance,
                                           const Color &c)
{
  root_map.cm[type].cm[instance].defaultColor = c;
}

Color AnnotationColors::instanceColor(const string &type,
                                      const string &instance) 
{
  return root_map.cm[type].cm[instance].defaultColor;
}

void AnnotationColors::erase(const string &type, 
                             const string& instance)
{
  root_map.cm[type].cm.erase(instance);
}

Color AnnotationColors::lookup(const annot &a) const
{
  return lookup(a.type, a.name);
}

Color AnnotationColors::lookup(const string &type, const string &instance) const
{
  // Yeah, there's probably a nicer looking recursive solution
  // to this problem.
  DefaultColorMap::const_iterator type_map = root_map.cm.find(type);
  if (type_map != root_map.cm.end() ) {
    // found lookup instance
    DefaultColorMap::const_iterator instance_map = type_map->second.cm.find(instance);
    if (instance_map != type_map->second.cm.end() ) {
      return instance_map->second.defaultColor;
    } else {
      return type_map->second.defaultColor;
    }
  } else {
    // not found
    return root_map.defaultColor;
  }
}

