#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLineEdit>
#include <QLabel>
#include <QFileDialog>
#include <QPushButton>

#include "qui/mussa_setup_dialog/SequenceSetupWidget.hpp"
#include "qui/mussa_setup_dialog/SequenceSetupFrame.hpp"

#include <cassert>

SequenceSetupWidget::SequenceSetupWidget(SequenceSetupFrame *setupFrame_,
                                         QWidget *parent) :
  QWidget(parent),
  sequenceNameEdit(0),
  sequenceLineEdit(0), 
  annotLineEdit(0),
  fastaIndexLineEdit(0),
  seqStartLineEdit(0),
  seqStopLineEdit(0),
  setupFrame(setupFrame_)
{
  // we really need the setupFrame, so lets declair that requirement
  assert(setupFrame != 0);

  sequenceNameEdit = new QLineEdit;
  QLabel *sequenceNameLabel = new QLabel(tr("Sequence name:"));
  sequenceNameLabel->setBuddy(sequenceNameEdit);

  // Sequence row
  sequenceLineEdit = new QLineEdit;
  //const IsFileValidator *fileValidator = new IsFileValidator(this);
  //sequenceLineEdit->setValidator(fileValidator);
  QLabel *sequenceLabel = new QLabel("Sequence Filename:");
  sequenceLabel->setBuddy(sequenceLineEdit);
  
  QPushButton *sequenceButton = new QPushButton("Browse...");
  connect(sequenceButton, SIGNAL(clicked()), 
	  this, SLOT(promptForFastaFile()));

  // Annotation row
  annotLineEdit = new QLineEdit;
  //const IsFileValidator *fileValidator2 = new IsFileValidator(this);
  //annotLineEdit->setValidator(fileValidator2);
  QLabel *annotLabel = new QLabel("Annotation Filename:");
  annotLabel->setBuddy(annotLineEdit);

  QPushButton *annotButton = new QPushButton("Browse...");
  connect(annotButton, SIGNAL(clicked()),
	  this, SLOT(promptForAnnotFile()));

  // Index row
  fastaIndexLineEdit = new QLineEdit;
  //fastaIndexLineEdit->setMaximumWidth(100);

  QLabel *fastaIndexLabel = new QLabel("Fasta Index:");
  fastaIndexLabel->setBuddy(fastaIndexLineEdit);

  seqStartLineEdit = new QLineEdit;
  //seqStartLineEdit->setMinimumWidth(60);
  //seqStartLineEdit->setMaximumWidth(80);

  QLabel *seqStartLabel = new QLabel("Sequence Start:");
  seqStartLabel->setBuddy(seqStartLineEdit);
  
  seqStopLineEdit = new QLineEdit;
  //seqStopLineEdit->setMinimumWidth(40);
  //seqStopLineEdit->setMaximumWidth(60);

  QLabel *seqStopLabel = new QLabel("Sequence Stop:");
  seqStopLabel->setBuddy(seqStopLineEdit);

  // Layout
  QHBoxLayout *sequenceNameLayout = new QHBoxLayout;
  QHBoxLayout *sequenceRowLayout = new QHBoxLayout;
  QHBoxLayout *annotRowLayout = new QHBoxLayout;
  QHBoxLayout *indexRowLayout = new QHBoxLayout;
  QVBoxLayout *mainLayout = new QVBoxLayout;

  sequenceNameLayout->addWidget(sequenceNameLabel);
  sequenceNameLayout->addWidget(sequenceNameEdit);

  sequenceRowLayout->addWidget(sequenceLabel);
  sequenceRowLayout->addWidget(sequenceLineEdit);
  sequenceRowLayout->addWidget(sequenceButton);

  annotRowLayout->addWidget(annotLabel);
  annotRowLayout->addWidget(annotLineEdit);
  annotRowLayout->addWidget(annotButton);

  indexRowLayout->addWidget(fastaIndexLabel);
  indexRowLayout->addWidget(fastaIndexLineEdit);
  indexRowLayout->addWidget(seqStartLabel);
  indexRowLayout->addWidget(seqStartLineEdit);
  indexRowLayout->addWidget(seqStopLabel);
  indexRowLayout->addWidget(seqStopLineEdit);

  mainLayout->addLayout(sequenceNameLayout);
  mainLayout->addLayout(sequenceRowLayout);
  mainLayout->addLayout(annotRowLayout);
  mainLayout->addLayout(indexRowLayout);
  setLayout(mainLayout);
}

//SequenceSetupWidget::SequenceSetupWidget(const SequenceSetupWidget& ssw) :
//  sequenceLineEdit(ssw.sequenceLineEdit), 
//  annotLineEdit(ssw.annotLineEdit),
//  fastaIndexLineEdit(ssw.fastaIndexLineEdit),
//  seqStartLineEdit(ssw.seqStartLineEdit),
//  seqStopLineEdit(ssw.seqStopLineEdit)
//{
//}

int SequenceSetupWidget::getFastaIndex() const
{
  if (fastaIndexLineEdit->text().size() == 0)
    return 1;
  else
    return fastaIndexLineEdit->text().toInt(); 
}

void SequenceSetupWidget::promptForFastaFile()
{
  QString default_dir = setupFrame->get_default_dir().absolutePath();
  QString fastaFilePath = QFileDialog::getOpenFileName(this,
					       "Choose a fasta file",
					       default_dir,
					       "Fasta (*.fa *.fasta *.fsa *.txt)");
  if (fastaFilePath.isEmpty())
    return;

  sequenceLineEdit->setText(fastaFilePath);
  setupFrame->set_default_dir(QDir(fastaFilePath));
  return;
}

void SequenceSetupWidget::promptForAnnotFile()
{
  QString default_dir = setupFrame->get_default_dir().absolutePath();
  QString annotFilePath = QFileDialog::getOpenFileName(this,
					       "Choose a annotation file",
					       default_dir,
					       "Annot (*.txt *.annot)");
  if (annotFilePath.isEmpty())
    return;

  annotLineEdit->setText(annotFilePath);
  setupFrame->set_default_dir(QDir(annotFilePath));
  return;
}
