#ifndef _SUBANALYSIS_H_
#define _SUBANALYSIS_H_

#include <QTableView>
#include <QPushButton>
#include <QSpinBox>
#include <QStringList>
#include <QWidget>

#include "qui/seqbrowser/seqproperties/SequencePropertiesModel.hpp"
#include "alg/mussa.hpp"

class PropertiesWindow : public QWidget
{
  Q_OBJECT 

public: 
  PropertiesWindow(/* MussaRef m, */
                   SequencePropertiesModel::model_type glseqs,
                   QWidget *parent = 0);

  //! return a modifiable reference to our SequenceLocationModel
  SequencePropertiesModel& getModel();

public slots:
  //! create a subanalysis and run it
  void apply();
  //! clear our model and close the window
  void close();
  //! provide a way for the model to tell us to update our gui
  void modelUpdated(const QModelIndex&, int, int);
  //! update our title
  void updateTitle();
  
signals:
  //! the user saved their changes
  void propertiesChanged();
  
private:
  //! keep track of what analysis we're attached to
  MussaRef analysis;
  QTableView *table;
  QPushButton *applyButton;
  QPushButton *closeButton;

  SequencePropertiesModel model;
};

//! reference to a subanalysis window
typedef boost::shared_ptr<PropertiesWindow> PropertiesWindowRef;
#endif
